% ECG telecharge de
% https://archive.physionet.org/cgi-bin/atm/ATM
% Echantillonne à 500Hz (F_Nyquist = 250Hz)
% Script OCTAVE (pas matlab...)
fid = fopen("values.txt", "w");

Fs = 500;     % Frequence d'echantillonnage
Fn = Fs/2;    % Frequence de Nyquist
%figure(1)
T = csvread('./ADCSamplesOctave.csv');
%subplot(2,3,1);plot(T(:,2));title('Raw ECG signal');xlabel('Samples (Fs=500Hz)');ylabel('Magnitude (output of an 11-bit ADC)');

% Pour Octave (à supprimer sous Matlab)
pkg load signal;

% Pour les trois filtres suivants, on peut jouer sur les ordres
% donc le nombre de coefficients des filtres numériques

% Suppression de la baseline (High-pass FIR)
fBaseLine = fir1(128, 5/Fn, 'high'); % Génération du filtre
y_minus_BL = filter(fBaseLine, [1], T(:,2)); % Application du filtre
fprintf(fid, "\n--- 1er Filtre ---\n");
for i = 1:length(y_minus_BL)
    fprintf(fid, "%f\n", y_minus_BL(i));
end

% subplot(2,3,2);plot(y_minus_BL);title('Baseline wander reduced');xlabel('Samples (Fs=500Hz)');ylabel('Magnitude (digital signal)');
% subplot(2,3,3);plot(y_minus_BL(1:1000));title('Baseline wander reduced -- zoomed');xlabel('Samples (Fs=500Hz)');ylabel('Magnitude (digital signal)');

% Élimination du bruit à 50Hz par un coupe-bande basique (Notch FIR)
f50Hz = fir1(100, [45 55]/Fn, 'stop'); % Génération du filtre
f50Hz = round(f50Hz * 2^10); % Normalisation et mise à l'échelle
y_minus_50Hz_simple = filter(f50Hz, [1], y_minus_BL); % Application du filtre

% subplot(2,3,4);plot(y_minus_50Hz_simple(1:1000));title('FIR1 band-cut-- zoomed');xlabel('Samples (Fs=500Hz)');ylabel('Magnitude (digital signal)');

% Élimination du bruit à 50Hz par un coupe-bande plus élaboré (Pei-Tseng) IIR
[b, a] = pei_tseng_notch(50 / Fn, 10/Fn);
y_minus_50Hz_pei_tseng = filter(b, a, y_minus_BL);
fprintf(fid, "\n--- 2eme Filtre ---\n");
for i = 1:length(y_minus_50Hz_pei_tseng)
    fprintf(fid, "%f\n", y_minus_50Hz_pei_tseng(i));
end

% subplot(2,3,5);plot(y_minus_50Hz_pei_tseng(1:1000));title('Pei Tseng band-cut -- zoomed');xlabel('Samples (Fs=500Hz)');ylabel('Magnitude (digital signal)');

% Lissage du bruit haute fréquence par filtre de Parks-McClellan (Low-pass FIR)
Fpass  = 50;
Fstop = 60;
F     = [0 Fpass Fstop Fn]/(Fn);
A     = [1 1 0 0];
fLP = remez(10, F, A); % Génération du filtre
yLP = filter(fLP, [1], y_minus_50Hz_pei_tseng); % Application du filtre
fprintf(fid, "\n--- 3eme Filtre ---\n");
for i = 1:length(yLP)
    fprintf(fid, "%f\n", yLP(i));
end

fclose(fid);

% subplot(2,3,6);plot(yLP(1:1000));title('Low-pass filter to suppress high-freq noise -- zoomed');xlabel('Samples (Fs=500Hz)');ylabel('Magnitude (digital signal)');
% figure(2)
% subplot(2,1,1);plot(T(:,2));title('Raw ECG signal');xlabel('Samples (Fs=500Hz)');ylabel('Magnitude (digital signal)');
% subplot(2,1,2);plot(yLP);title('After 3 filters');xlabel('Samples (Fs=500Hz)');ylabel('Magnitude (digital signal)');
% print(2, "ECG_raw_3filters.pdf", "-dpdflatexstandalone");

% figure(3)

% L'artillerie lourde: fonction intégrant la méthode de Pan-Tompkin
% Merci Sedghamiz. H !!!
%pan_tompkin(T(:,2),500,1);

% EXPORT DES COEFFICIENTS POUR VHDL
% Conversion en entiers signés et exportation
printf("\n--- Coefficients du filtre Baseline ---\n");
fid = fopen("coeficientes_vhdl.txt", "w");

fprintf(fid, "\n--- Coefficients du filtre Baseline ---\n");
for i = 1:length(fBaseLine)
    fprintf(fid, "to_signed(%d, 12),\n", fBaseLine(i));
end

%fprintf(fid, "\n--- Coefficients du filtre Notch 50Hz ---\n");
%for i = 1:length(f50Hz)
%    fprintf(fid, "to_signed(%d, 11),\n", f50Hz(i));
%end

fprintf(fid, "\n--- Coefficients du filtre Pei-Tseng A ---\n");
for i = 1:length(a)
    fprintf(fid, "to_signed(%d, 12),\n", a(i));
end

fprintf(fid, "\n--- Coefficients du filtre Pei-Tseng B ---\n");
for i = 1:length(b)
    fprintf(fid, "to_signed(%d, 12),\n", b(i));
end

fprintf(fid, "\n--- Coefficients du filtre Parks-McClellan ---\n");
for i = 1:length(fLP)
    fprintf(fid, "to_signed(%d, 12),\n", fLP(i));
end

fclose(fid);
printf("\nThe coefficients were saved in 'coeficientes_vhdl.txt'.\n");
