-- filepath: c:\Users\Conta\Documents\GitHub\IMT\tp-ecg-etudiant-a24perei\src\hdl\controlUnit.vhd
LIBRARY ieee;
USE ieee.std_logic_1164.ALL;
USE ieee.numeric_std.ALL;

ENTITY controlUnit IS
  PORT (
    I_clock : IN STD_LOGIC;
    I_reset : IN STD_LOGIC;
    I_inputSampleValid : IN STD_LOGIC;
    I_processingDoneA : IN STD_LOGIC;
    I_processingDoneB : IN STD_LOGIC;
    I_processingDoneBL : IN STD_LOGIC;
    I_processingDoneClellan : IN STD_LOGIC;
    O_loadShift : OUT STD_LOGIC;
    O_initAddress : OUT STD_LOGIC;
    O_incrAddress : OUT STD_LOGIC;
    O_initSum : OUT STD_LOGIC;
    O_loadSum : OUT STD_LOGIC;
    O_loadOutA : OUT STD_LOGIC;
    O_loadOutB : OUT STD_LOGIC;
    O_loadOutBL : OUT STD_LOGIC;
    O_loadOutClellan : OUT STD_LOGIC;
    O_sel : OUT STD_LOGIC_VECTOR(1 DOWNTO 0);
    O_FilteredSampleValid : OUT STD_LOGIC
  );
END ENTITY controlUnit;

ARCHITECTURE archi_operativeUnit OF controlUnit IS
  TYPE T_state IS (WAIT_SAMPLE, STORE, BASELINE, PROCESS_A, PROCESS_B, CLELLAN, OUTPUT, WAIT_END_SAMPLE);
  SIGNAL SR_presentState : T_state := WAIT_SAMPLE;
  SIGNAL SR_futurState : T_state := WAIT_SAMPLE;
BEGIN

  -- State transition process
  PROCESS (I_reset, I_clock)
  BEGIN
    IF I_reset = '1' THEN
      SR_presentState <= WAIT_SAMPLE;
    ELSIF rising_edge(I_clock) THEN
      SR_presentState <= SR_futurState;
    END IF;
  END PROCESS;

  -- Next state logic and output control
  PROCESS (SR_presentState, I_inputSampleValid, I_processingDoneBL, I_processingDoneClellan, I_processingDoneA, I_processingDoneB)
  BEGIN
    -- Default values for outputs
    O_sel <= "00";
    O_initAddress <= '0';
    O_initSum <= '0';

    -- State machine
    CASE SR_presentState IS
      WHEN WAIT_SAMPLE =>
        IF I_inputSampleValid = '1' THEN
          SR_futurState <= STORE;
        ELSE
          SR_futurState <= WAIT_SAMPLE;
        END IF;

      WHEN STORE =>
        O_initAddress <= '1';
        O_sel <= "00";
        O_initSum <= '1';
        SR_futurState <= BASELINE;

      WHEN BASELINE =>
        O_sel <= "00";
        IF I_processingDoneBL = '1' THEN
          SR_futurState <= PROCESS_B;
          O_initAddress <= '1';
          O_initSum <= '1';
        ELSE
          SR_futurState <= BASELINE;
          O_initAddress <= '0';
          O_initSum <= '0';
        END IF;

      WHEN PROCESS_B =>
        O_sel <= "01";
        O_initAddress <= '0';
        IF I_processingDoneB = '1' THEN
          SR_futurState <= PROCESS_A;
          O_initAddress <= '1';
          O_initSum <= '1';
        ELSE
          SR_futurState <= PROCESS_B;
          O_initAddress <= '0';
          O_initSum <= '0';
        END IF;

      WHEN PROCESS_A =>
        O_initAddress <= '0';
        O_sel <= "10";
        
        IF I_processingDoneA = '1' THEN
          SR_futurState <= CLELLAN;
          O_initAddress <= '1';
          O_initSum <= '1';
        ELSE
          SR_futurState <= PROCESS_A;
          O_initSum <= '0';
        END IF;

      WHEN CLELLAN =>
        O_sel <= "11";
        O_initAddress <= '0';
        IF I_processingDoneClellan = '1' THEN
          SR_futurState <= OUTPUT;
          O_initSum <= '1';
        ELSE
          SR_futurState <= CLELLAN;
          O_initSum <= '0';
        END IF;

      WHEN OUTPUT =>
        SR_futurState <= WAIT_END_SAMPLE;

      WHEN WAIT_END_SAMPLE =>
        IF I_inputSampleValid = '0' THEN
          SR_futurState <= WAIT_SAMPLE;
        ELSE
          SR_futurState <= WAIT_END_SAMPLE;
        END IF;

      WHEN OTHERS =>
        SR_futurState <= WAIT_SAMPLE;
    END CASE;
  END PROCESS;

  -- Output logic
  O_loadShift <= '1' WHEN SR_presentState = STORE ELSE
    '0';
  O_incrAddress <= '1' WHEN SR_presentState = BASELINE OR SR_presentState = PROCESS_A OR SR_presentState = PROCESS_B OR SR_presentState = CLELLAN ELSE
    '0';
  O_loadSum <= '1' WHEN SR_presentState = BASELINE OR SR_presentState = PROCESS_A OR SR_presentState = PROCESS_B OR SR_presentState = CLELLAN ELSE
    '0';
  O_loadOutBL <= '1' WHEN SR_presentState = BASELINE ELSE
    '0';
  O_loadOutA <= '1' WHEN SR_presentState = PROCESS_A ELSE
    '0';
  O_loadOutB <= '1' WHEN SR_presentState = PROCESS_B ELSE
    '0';
  O_loadOutClellan <= '1' WHEN SR_presentState = OUTPUT ELSE
    '0';
  O_FilteredSampleValid <= '1' WHEN SR_presentState = OUTPUT ELSE
    '0';

END ARCHITECTURE archi_operativeUnit;