LIBRARY ieee;
USE ieee.std_logic_1164.ALL;
USE ieee.numeric_std.ALL;

ENTITY operativeUnit IS

  PORT (
    I_clock : IN STD_LOGIC;
    I_reset : IN STD_LOGIC;
    I_inputSample : IN STD_LOGIC_VECTOR(11 DOWNTO 0);
    I_loadShift : IN STD_LOGIC;
    I_initAddress : IN STD_LOGIC;
    I_incrAddress : IN STD_LOGIC;
    I_sel : IN STD_LOGIC_VECTOR(1 DOWNTO 0);
    I_initSum : IN STD_LOGIC;
    I_loadSum : IN STD_LOGIC;
    I_loadOutA : IN STD_LOGIC;
    I_loadOutB : IN STD_LOGIC;
    I_loadOutBL : IN STD_LOGIC;
    I_loadOutClellan : IN STD_LOGIC;
    O_processingDoneA : OUT STD_LOGIC;
    O_processingDoneB : OUT STD_LOGIC;
    O_processingDoneBL : OUT STD_LOGIC;
    O_processingDoneClellan : OUT STD_LOGIC;
    O_total : OUT STD_LOGIC_VECTOR(11 DOWNTO 0)
  );

END ENTITY operativeUnit;

ARCHITECTURE arch_operativeUnit OF operativeUnit IS
  TYPE registerFileA IS ARRAY(0 TO 2) OF signed(11 DOWNTO 0);
  SIGNAL SR_coefRegisterA : registerFileA;
  SIGNAL SR_2A : registerFileA;

  TYPE registerFileB IS ARRAY(0 TO 2) OF signed(11 DOWNTO 0);
  SIGNAL SR_coefRegisterB : registerFileB;
  SIGNAL SR_2 : registerFileB;

  TYPE registerFileBL IS ARRAY(0 TO 94) OF signed(11 DOWNTO 0);
  SIGNAL SR_coefRegisterBL : registerFileBL;
  SIGNAL SR_1 : registerFileBL;

  TYPE registerFileClellan IS ARRAY(0 TO 10) OF signed(11 DOWNTO 0);
  SIGNAL SR_coefRegisterClellan : registerFileClellan;
  SIGNAL SR_3 : registerFileClellan;

  SIGNAL SC_multOperand1 : signed(11 DOWNTO 0);
  SIGNAL SC_multOperand2 : signed(11 DOWNTO 0);
  SIGNAL SC_MultResult : signed(23 DOWNTO 0);
  SIGNAL SC_addResult : signed(30 DOWNTO 0);
  SIGNAL SR_sum : signed(30 DOWNTO 0);
  SIGNAL SR_Y : signed(11 DOWNTO 0);
  SIGNAL SR_readAddress : INTEGER RANGE 0 TO 94;

BEGIN
  SR_coefRegisterA <= (to_signed(-1024, 12),
    to_signed(1554, 12),
    to_signed(-897, 12)
    );

  SR_coefRegisterB <= (to_signed(961, 12),
    to_signed(-1554, 12),
    to_signed(961, 12)
    );

  SR_coefRegisterBL <= (to_signed(-1, 12),
    to_signed(-1, 12),
    to_signed(-1, 12),
    to_signed(-1, 12),
    to_signed(-1, 12),
    to_signed(-1, 12),
    to_signed(-2, 12),
    to_signed(-2, 12),
    to_signed(-2, 12),
    to_signed(-3, 12),
    to_signed(-3, 12),
    to_signed(-3, 12),
    to_signed(-4, 12),
    to_signed(-4, 12),
    to_signed(-5, 12),
    to_signed(-5, 12),
    to_signed(-6, 12),
    to_signed(-6, 12),
    to_signed(-7, 12),
    to_signed(-7, 12),
    to_signed(-8, 12),
    to_signed(-8, 12),
    to_signed(-9, 12),
    to_signed(-10, 12),
    to_signed(-10, 12),
    to_signed(-11, 12),
    to_signed(-11, 12),
    to_signed(-12, 12),
    to_signed(-13, 12),
    to_signed(-13, 12),
    to_signed(-14, 12),
    to_signed(-14, 12),
    to_signed(-15, 12),
    to_signed(-15, 12),
    to_signed(-16, 12),
    to_signed(-16, 12),
    to_signed(-17, 12),
    to_signed(-17, 12),
    to_signed(-18, 12),
    to_signed(-18, 12),
    to_signed(-18, 12),
    to_signed(-19, 12),
    to_signed(-19, 12),
    to_signed(-19, 12),
    to_signed(-19, 12),
    to_signed(-19, 12),
    to_signed(-19, 12),
    to_signed(1004, 12),
    to_signed(-19, 12),
    to_signed(-19, 12),
    to_signed(-19, 12),
    to_signed(-19, 12),
    to_signed(-19, 12),
    to_signed(-19, 12),
    to_signed(-18, 12),
    to_signed(-18, 12),
    to_signed(-18, 12),
    to_signed(-17, 12),
    to_signed(-17, 12),
    to_signed(-16, 12),
    to_signed(-16, 12),
    to_signed(-15, 12),
    to_signed(-15, 12),
    to_signed(-14, 12),
    to_signed(-14, 12),
    to_signed(-13, 12),
    to_signed(-13, 12),
    to_signed(-12, 12),
    to_signed(-11, 12),
    to_signed(-11, 12),
    to_signed(-10, 12),
    to_signed(-10, 12),
    to_signed(-9, 12),
    to_signed(-8, 12),
    to_signed(-8, 12),
    to_signed(-7, 12),
    to_signed(-7, 12),
    to_signed(-6, 12),
    to_signed(-6, 12),
    to_signed(-5, 12),
    to_signed(-5, 12),
    to_signed(-4, 12),
    to_signed(-4, 12),
    to_signed(-3, 12),
    to_signed(-3, 12),
    to_signed(-3, 12),
    to_signed(-2, 12),
    to_signed(-2, 12),
    to_signed(-2, 12),
    to_signed(-1, 12),
    to_signed(-1, 12),
    to_signed(-1, 12),
    to_signed(-1, 12),
    to_signed(-1, 12),
    to_signed(-1, 12)
    );

  SR_coefRegisterClellan <= (to_signed(-119, 12),
    to_signed(122, 12),
    to_signed(149, 12),
    to_signed(191, 12),
    to_signed(226, 12),
    to_signed(239, 12),
    to_signed(226, 12),
    to_signed(191, 12),
    to_signed(149, 12),
    to_signed(122, 12),
    to_signed(-119, 12)
    );

  shift : PROCESS (I_reset, I_clock) IS
  BEGIN
    IF I_reset = '1' THEN
      SR_1 <= (OTHERS => (OTHERS => '0'));

    ELSIF rising_edge(I_clock) THEN
      IF I_loadShift = '1' THEN
        IF I_sel = "00" THEN
          SR_1(1 TO 94) <= SR_1(0 TO 93);
          SR_1(0) <= signed(I_inputSample);
        END IF;
      END IF;
    END IF;
  END PROCESS shift;

  incr_address : PROCESS (I_reset, I_clock) IS
  BEGIN
    IF I_reset = '1' THEN
      SR_readAddress <= 0;

    ELSIF rising_edge(I_clock) THEN
      IF I_sel = "00" THEN
        IF I_initAddress = '1' THEN
          SR_readAddress <= 0;
        ELSIF I_incrAddress = '1' THEN
          IF SR_readAddress = 94 THEN
            SR_readAddress <= 0;
          ELSE
            SR_readAddress <= SR_readAddress + 1;
          END IF;
        END IF;

        -- Process B
      ELSIF I_sel = "01" THEN
        IF I_initAddress = '1' THEN
          SR_readAddress <= 0;
        ELSIF I_incrAddress = '1' THEN
          IF SR_readAddress = 2 THEN
            SR_readAddress <= 0;
          ELSE
            SR_readAddress <= SR_readAddress + 1;
          END IF;
        END IF;

        -- Process A
      ELSIF I_sel = "10" THEN
        IF I_initAddress = '1' THEN
          SR_readAddress <= 0;
        ELSIF I_incrAddress = '1' THEN
          IF SR_readAddress = 2 THEN
            SR_readAddress <= 0;
          ELSE
            SR_readAddress <= SR_readAddress + 1;
          END IF;
        END IF;

        -- Process Clellan
      ELSIF I_sel = "11" THEN
        IF I_initAddress = '1' THEN
          SR_readAddress <= 0;
        ELSIF I_incrAddress = '1' THEN
          IF SR_readAddress = 10 THEN
            SR_readAddress <= 0;
          ELSE
            SR_readAddress <= SR_readAddress + 1;
          END IF;
        END IF;
      END IF;
    END IF;

  END PROCESS incr_address;

  mux_operands : PROCESS (SR_readAddress, I_sel, SR_1, SR_coefRegisterBL,
    SR_3, SR_coefRegisterClellan, SR_2,
    SR_coefRegisterB, SR_2A, SR_coefRegisterA) IS
  BEGIN
    IF I_sel = "00" THEN
      SC_multOperand1 <= SR_1(SR_readAddress);
      SC_multOperand2 <= SR_coefRegisterBL(SR_readAddress);

      IF SR_readAddress = 94 THEN
        O_processingDoneBL <= '1';
        O_processingDoneA <= '0';
        O_processingDoneB <= '0';
        O_processingDoneClellan <= '0';
      ELSE
        O_processingDoneA <= '0';
        O_processingDoneB <= '0';
        O_processingDoneBL <= '0';
        O_processingDoneClellan <= '0';
      END IF;

    ELSIF I_sel = "01" THEN
      SC_multOperand1 <= SR_2(SR_readAddress);
      SC_multOperand2 <= SR_coefRegisterB(SR_readAddress);

      IF SR_readAddress = 2 THEN
        O_processingDoneB <= '1';
        O_processingDoneA <= '0';
        O_processingDoneBL <= '0';
        O_processingDoneClellan <= '0';
      ELSE
        O_processingDoneA <= '0';
        O_processingDoneB <= '0';
        O_processingDoneBL <= '0';
        O_processingDoneClellan <= '0';
      END IF;

    ELSIF I_sel = "10" THEN
      SC_multOperand1 <= SR_2A(SR_readAddress);
      SC_multOperand2 <= SR_coefRegisterA(SR_readAddress);

      IF SR_readAddress = 1 THEN
        O_processingDoneA <= '1';
        O_processingDoneB <= '0';
        O_processingDoneBL <= '0';
        O_processingDoneClellan <= '0';
      ELSE
        O_processingDoneA <= '0';
        O_processingDoneB <= '0';
        O_processingDoneBL <= '0';
        O_processingDoneClellan <= '0';
      END IF;

    ELSIF I_sel = "11" THEN
      SC_multOperand1 <= SR_3(SR_readAddress);
      SC_multOperand2 <= SR_coefRegisterClellan(SR_readAddress);

      IF SR_readAddress = 10 THEN
        O_processingDoneClellan <= '1';
        O_processingDoneB <= '0';
        O_processingDoneA <= '0';
        O_processingDoneBL <= '0';
      ELSE
        O_processingDoneA <= '0';
        O_processingDoneB <= '0';
        O_processingDoneBL <= '0';
        O_processingDoneClellan <= '0';
      END IF;
    END IF;
  END PROCESS mux_operands;

  SC_MultResult <= SC_multOperand1 * SC_multOperand2;
  SC_addResult <= resize(SC_MultResult, SC_addResult'length) + SR_sum;

  sum_acc : PROCESS (I_reset, I_clock) IS
  BEGIN
    IF I_reset = '1' THEN
      SR_sum <= (OTHERS => '0');

    ELSIF rising_edge(I_clock) THEN
      IF I_initSum = '1' THEN
        SR_sum <= (OTHERS => '0');
      ELSIF I_loadSum = '1' THEN
        SR_sum <= SC_addResult;
      END IF;

    END IF;
  END PROCESS sum_acc;

  store_result : PROCESS (I_reset, I_clock) IS
  BEGIN
    IF I_reset = '1' THEN
      SR_Y <= (OTHERS => '0');
      SR_2A <= (OTHERS => (OTHERS => '0'));
      SR_2 <= (OTHERS => (OTHERS => '0'));
      SR_3 <= (OTHERS => (OTHERS => '0'));

    ELSIF rising_edge(I_clock) THEN
      IF I_loadOutBL = '1' THEN
        IF SC_addResult(9) = '1' THEN
          SR_2(1 TO 2) <= SR_2(0 TO 1);
          SR_2(0) <= signed(SC_addResult(21 DOWNTO 10) + 1);
        ELSE
          SR_2(1 TO 2) <= SR_2(0 TO 1);
          SR_2(0) <= signed(SC_addResult(21 DOWNTO 10));
        END IF;

      ELSIF I_loadOutB = '1' THEN
      IF SC_addResult(9) = '1' THEN
        SR_2A(1 TO 2) <= SR_2A(0 TO 1);
        SR_2A(0) <= signed(SC_addResult(21 DOWNTO 10) + 1);
      ELSE
        SR_2A(1 TO 2) <= SR_2A(0 TO 1);
        SR_2A(0) <= signed(SC_addResult(21 DOWNTO 10));
      END IF;

      ELSIF I_loadOutA = '1' THEN
        SR_3(1 TO 10) <= SR_3(0 TO 9);
        IF SC_addResult(9) = '1' THEN
          SR_3(0) <= signed(SC_addResult(21 DOWNTO 10)) + 1;
        ELSE
          SR_3(0) <= signed(SC_addResult(21 DOWNTO 10));
        END IF;
        
      ELSIF I_loadOutClellan = '1' THEN
        IF SC_addResult(9) = '1' THEN
          SR_Y <= SC_addResult(21 DOWNTO 10) + 1;
        ELSE
          SR_Y <= SC_addResult(21 DOWNTO 10);
        END IF;
      END IF;
    END IF;
  END PROCESS store_result;

  O_total <= STD_LOGIC_VECTOR(SR_Y);

END ARCHITECTURE arch_operativeUnit;